-- DIM 2.1.0.applescript
-- DIM 2.1.0

--  Created by G.J. Parker on 2012/1/8.
--  Copyright 2012. All rights reserved.

(* this applescript studio application records the configurations of positions of the icons on the desktop along w/ the screen
resolution in the plist file ~/Library/Preferences/com.parker9.DIM_2.1.plist. it also allows the user to have the code
automatically restore the icon positions and quit. to get back to memorizing and/or selecting the auto-restore, the 
user needs to drag and drop anything onto the application. finally, if the screen resolution is different, we simply place the icons
in the same relative positions as initially set *)

(* set properties *)
property IconsAll : {} (* list of all icons on desktop *)
property PositionAll : {} (* corresponding list of positions of the icons on desktop *)
property ScreenSize : {} (* screem size for the previous lists *)
property prompt_p : true (* should user see main window, or default to just 'Restore' and quit? *)
property SavedConfigs : {} (* list of all saved configurations *)
property DefaultConfig : "" (* default configuration name *)
property start_p : true (* true when first started, but quickly false *)
property EditWindow : missing value (* handle for Edit panel *)
property DefaultName : "" (* Restore's default configuration name *)
property EditName : "" (* string for Save popup menu, "Edit..." in english *)
property SelectedConfig : missing value
property icon_arrangement_text : ""

on open names
	set start_p to false (* things were dropped on us, so don't just restore and quit! *)
end open

on activated theObject
	if prompt_p = false and start_p = true then (* if restore and quit and we just started w/o any drag and drop, then restore and quit! *)
		restore() of me
		quit
	else
		tell application "Finder" to set x to count of (items of desktop)
		tell window "DIM" to set contents of text field "num_current" to (x as string)
	end if
	set start_p to false (* we just didn't just start or we did, but we had drag and dropped on us *)
end activated

on should quit after last window closed theObject -- quit when the window is closed
	quit
end should quit after last window closed


on will finish launching theObject (* this happens after we start *)
	(* we have to check that they are running tiger, otherwise the script won't work! blame Apple *)
	if (system attribute "sysv") < 4160 then (* they need Tiger- 4160 converted to hex is 1040 = 10.4.0 *)
		set request to display dialog "Desktop Icon Manager 2.0 only works for Mac OS X 10.4 and later" buttons {"Quit"} default button "Quit"
		quit
	else
		(* there's a difference in applescript and applescript studio- properties are not persisent! so, you store these things in a preference file... *)
		make new default entry at end of default entries of user defaults with properties {name:"SavedConfigs", contents:{}}
		make new default entry at end of default entries of user defaults with properties {name:"DefaultConfig", contents:" "}
		make new default entry at end of default entries of user defaults with properties {name:"prompt_p", contents:true}
		set SavedConfigs to contents of default entry "SavedConfigs" of user defaults (* load up values *)
		set DefaultConfig to contents of default entry "DefaultConfig" of user defaults
		set prompt_p to contents of default entry "prompt_p" of user defaults
		
		tell window "DIM"
			set DefaultName to title of menu item 2 of popup button "arrangement_pop" (* default root name of new configurations *)
			set EditName to title of last menu item of popup button "arrangement_pop" (* name of "Edit..." for save popup menu *)
			set icon_arrangement_text to contents of text field "arrangement_text"
			(*set icon_arrangement_text to "Use icon arrangment: "*)
		end tell
		
		if (count of SavedConfigs) = 0 then (* first time run, save a default configuration *)
			copy DefaultName to DefaultConfig
			memorize(DefaultConfig) of me
		else
			getConfig(DefaultConfig) (* otherwise load up Restore's default configuration *)
		end if
		populatePopup(DefaultConfig) (* populate Save and Restore popup menues *)
		
		tell window "DIM" (* set up window elements *)
			if prompt_p = true then
				set state of button "auto_restore" to 0
				set visible of text field "hint" to false
			else
				set state of button "auto_restore" to 1
				set visible of text field "hint" to true
			end if
			set contents of text field "num_memorized" to ((count of IconsAll) as string)
			tell application "Finder" to set x to count of (items of desktop)
			set contents of text field "num_current" to (x as string)
			set visible of text field "arrangement_text" to true
			set visible of text field "num_mem_text" to true
			set visible of text field "num_current_text" to true
			set visible of text field "num_memorized" to true
			set visible of text field "num_current" to true
		end tell
	end if
end will finish launching

on clicked theObject (* something was clicked on *)
	if name of theObject is "auto_restore" then (* user is toggling the flag for automatically restore *)
		tell window "DIM"
			set a to state of button "auto_restore"
			if state of button "auto_restore" is 1 then
				set prompt_p to false
				set visible of text field "hint" to true
			else
				set prompt_p to true
				set visible of text field "hint" to false
			end if
		end tell
		set contents of default entry "prompt_p" of user defaults to prompt_p (* and store it so it's persisent *)
	else if name of theObject is "restore" then (* user wants to restore *)
		restore() of me
	else if name of theObject is "memorize" then (* user wants to memorize *)
		memorize(DefaultConfig) of me
	else if name of theObject is "done" then (* close the Edit panel *)
		(*if SelectedConfig is not equal to missing value then set DefaultConfig to SelectedConfig*)
		getConfig(DefaultConfig)
		populatePopup(DefaultConfig) (* update the popdown menue *)
		close panel (window of theObject)
	else if name of theObject is "add" then (* add a new configuration *)
		set newName to DefaultName (* find a unique configuration name *)
		set i to 0
		repeat while (item 1 of SavedConfigs contains newName)
			set i to i + 1
			set newName to DefaultName & " " & i
		end repeat
		display dialog "Name of new Icon Arrangement:" default answer newName buttons {"Cancel", "OK"} default button "OK" attached to window of theObject
	else if name of theObject is "delete" then (* delete the current selection *)
		deleteConfig(SelectedConfig)
	end if
	tell application "Finder" to set x to count of (items of desktop)
	tell window "DIM" to set contents of text field "num_current" to (x as string)
end clicked

on choose menu item theObject (* handles either main menu or popup button menu selection *)
	if (name of theObject) is "close" then (* Quit from main menu selected *)
		quit
	else (* the popdown button menu was selected *)
		if name of theObject is "arrangement_pop" then (* save popup... *)
			set x to title of current menu item of popup button "arrangement_pop" of the window of theObject
			if x = EditName then (* want to Edit configurations *)
				(*	if EditWindow is equal to missing value then  *)
				load nib "EditSheet" (* load up Edit sheet... *)
				set EditWindow to window "edit_sheet"
				register drag types {"rows"}
				(*	end if *)
				set SelectedConfig to missing value
				tell EditWindow (* set Edit panel in a defined state *)
					set enabled of button "delete" to false
					set content of table view "table" of scroll view "scroll" to item 1 of SavedConfigs
					set allows reordering of table view "table" of scroll view "scroll" to true
				end tell
				--		display panel EditWindow attached to window "DIM"
				display EditWindow attached to window "DIM"
			else (* chose a configuration *)
				set DefaultConfig to x
				getConfig(DefaultConfig)
				populatePopup(DefaultConfig) (* over kill, but we need to put a check mark on popdown's configuration *)
			end if
		end if
		tell application "Finder" to set x to count of (items of desktop)
		tell window "DIM" to set contents of text field "num_current" to (x as string)
	end if
end choose menu item



on selection changed theObject (* more than two configurations saved and we're on Edit panel, user can delete if they select a configuration *)
	
	if name of theObject is "table" then
		set mySelection to selected data rows of theObject
		if length of mySelection > 0 then (* is something actually selected? *)
			repeat with thisRow in mySelection
				set SelectedConfig to (the contents of data cell 1 of thisRow) as text
			end repeat
			tell EditWindow (* set the 'delete' button to the correct state *)
				if (count of item 1 of SavedConfigs) > 1 and SelectedConfig is not equal to missing value then
					set enabled of button "delete" to true
				else
					set enabled of button "delete" to false
				end if
			end tell
		else
			set SelectedConfig to missing value
			tell EditWindow
				set enabled of button "delete" to false
				set enabled of button "add" to true
				set enabled of button "done" to true
			end tell
		end if
	end if
end selection changed

on double clicked theObject
	local tRow, tCol, tDat, theReply
	
	if the class of theObject is table view then
		if the name of theObject = "table" then
			try
				set tRow to selected data row of theObject
				set tDat to contents of data cell "property" of tRow
				set theReply to display dialog "Enter new Icon Arrangement name:" buttons {"Cancel", "OK"} default button "OK" default answer tDat
				if button returned of theReply is equal to "OK" then
					set newName to text returned of theReply
					if newName is not equal to tDat then
						set the contents of data cell "property" of tRow to newName
						if DefaultConfig is not equal to newName then
							set DefaultConfig to newName
							set contents of default entry "DefaultConfig" of user defaults to DefaultConfig
						end if
						repeat with j from 1 to count (item 1 of SavedConfigs)
							if item j of item 1 of SavedConfigs = tDat then
								set item j of item 1 of SavedConfigs to newName
								set contents of default entry "SavedConfigs" of user defaults to SavedConfigs
							end if
						end repeat
						
					end if
				end if
			end try
		end if
	end if
end double clicked
on rows changed theObject (* this allows user to adjust the order of configurations *)
	tell EditWindow (* freeze interface so nothing else can happen *)
		set enabled of button "done" to false
		set enabled of table view "table" of scroll view "scroll" to false
		set enabled of button "delete" to false
		set enabled of button "add" to false
	end tell
	set newOrder to contents of every data cell of every data row of data source of table view "table" of scroll view "scroll" of EditWindow (* new order *)
	copy SavedConfigs to OldSavedConfigs
	set SavedConfigs to {}
	repeat with i from 1 to count of (item 1 of OldSavedConfigs) (* loop over new order *)
		repeat with j from 1 to count of (item 1 of OldSavedConfigs) (* find index in old order *)
			if item 1 of item i of newOrder = item j of item 1 of OldSavedConfigs then
				if i = 1 then (* special case for first configuration *)
					set SavedConfigs to {item i of newOrder, {item j of item 2 of OldSavedConfigs}, {item j of item 3 of OldSavedConfigs}, {item j of item 4 of OldSavedConfigs}}
				else
					copy item 1 of item i of newOrder to the end of (item 1 of SavedConfigs)
					copy item j of item 2 of OldSavedConfigs to the end of (item 2 of SavedConfigs)
					copy item j of item 3 of OldSavedConfigs to the end of (item 3 of SavedConfigs)
					copy item j of item 4 of OldSavedConfigs to the end of (item 4 of SavedConfigs)
				end if
			end if
		end repeat
	end repeat
	set contents of default entry "SavedConfigs" of user defaults to SavedConfigs --store them, damn applescript studio
	tell EditWindow
		set enabled of button "done" to true
		set enabled of table view "table" of scroll view "scroll" to true
		set enabled of button "add" to true
		set content of table view "table" of scroll view "scroll" to item 1 of SavedConfigs
	end tell
end rows changed

on dialog ended theObject with reply theReply (*this handles adding a new configuration *)
	if button returned of theReply is "OK" then (* do we really want to store a new configuration? *)
		if (item 1 of SavedConfigs contains text returned of theReply) then (* name must be unique *)
			beep
		else
			tell EditWindow (* freeze interface so nothing else can happen *)
				set visible of text field "memorize_message" to true
				set visible of progress indicator "spinning" to true
				set enabled of button "done" to false
				set enabled of table view "table" of scroll view "scroll" to false
				set enabled of button "delete" to false
				set enabled of button "add" to false
				start progress indicator "spinning"
			end tell
			set x to text returned of theReply
			memorize(x) of me (* do the actual save *)
			set DefaultConfig to x
			tell EditWindow (* now go back and show the Edit panel *)
				set enabled of button "done" to true
				set enabled of table view "table" of scroll view "scroll" to true
				set enabled of button "add" to true
				set visible of text field "memorize_message" to false
				stop progress indicator "spinning"
				set visible of progress indicator "spinning" to false
				set enabled of button "delete" to false
				set content of table view "table" of scroll view "scroll" to item 1 of SavedConfigs
			end tell
		end if
	end if
end dialog ended

on restore() -- restore icon positions
	tell window "DIM" -- setup up window
		set enabled of popup button "arrangement_pop" to false
		set enabled of button "auto_restore" to false
		set visible of text field "restore_message" to true
		set visible of progress indicator "spinning" to true
		start progress indicator "spinning"
	end tell
	tell application "Finder" -- now onto the actual restore positions
		copy the bounds of the desktop's window to the NewScreenSize -- get current resolution
		if arrangement of (icon view options of desktop's window) ≠ not arranged and NewScreenSize ≠ ScreenSize then
			set arrangement of (icon view options of desktop's window) to not arranged -- we have to turn off Snap to Grid or whatever else if resolution changed
		end if
		
		set newdx to item 1 of NewScreenSize -- scaling parameters
		set newdy to item 2 of NewScreenSize
		set xrel to ((item 3 of NewScreenSize) - newdx) / ((item 3 of ScreenSize) - (item 1 of ScreenSize))
		set yrel to ((item 4 of NewScreenSize) - newdy) / ((item 4 of ScreenSize) - (item 2 of ScreenSize))
		repeat with i from 1 to number of items of IconsAll -- for each desktop icon
			ignoring application responses
				tell application "Finder" to set desktop position of item (item i of IconsAll) of desktop to ¬
					{newdx + (0.5 + ((item 1 of (item i of PositionAll)) - newdx) * xrel) div 1, ¬
						newdy + (0.5 + ((item 2 of (item i of PositionAll)) - newdy) * yrel) div 1} -- new position
				
			end ignoring
		end repeat
	end tell
	tell window "DIM" -- reset window
		set enabled of popup button "arrangement_pop" to true
		set enabled of button "auto_restore" to true
		set visible of text field "restore_message" to false
		set visible of progress indicator "spinning" to false
		stop progress indicator "spinning"
		tell application "Finder" to set x to count of (items of desktop)
		set contents of text field "num_current" to (x as string)
		set visible of text field "num_memorized" to true
		set visible of text field "num_current" to true
	end tell
end restore

on memorize(thisConfig) -- memorize icon positions to configuration thisConfig
	tell window "DIM" -- set up window
		set visible of text field "memorize_message" to true
		set enabled of popup button "arrangement_pop" to false
		set enabled of button "auto_restore" to false
		set visible of text field "arrangement_text" to false
		set visible of text field "num_memorized" to false
		set visible of text field "num_current" to false
		set visible of text field "num_mem_text" to false
		set visible of text field "num_current_text" to false
		set visible of progress indicator "spinning" to true
		start progress indicator "spinning"
	end tell
	copy {} to IconsAll -- zero out everything
	copy {} to PositionAll
	copy {} to ScreenSize
	tell application "Finder" -- go get names, positions and current screen size
		set IconsAll to name of items of desktop
		set PositionAll to desktop position of items of desktop
		copy the bounds of the desktop's window to the ScreenSize
	end tell
	saveConfig(thisConfig)
	tell window "DIM" --reset window
		set enabled of popup button "arrangement_pop" to true
		set enabled of button "auto_restore" to true
		set visible of text field "memorize_message" to false
		set visible of progress indicator "spinning" to false
		stop progress indicator "spinning"
		set contents of text field "num_memorized" to ((count of IconsAll) as string)
		set contents of text field "num_current" to ((count of IconsAll) as string)
		set visible of text field "arrangement_text" to true
		set visible of text field "num_mem_text" to true
		set visible of text field "num_current_text" to true
		set visible of text field "num_memorized" to true
		set visible of text field "num_current" to true
	end tell
end memorize

on getConfig(thisConfig) -- load up IconsAll, PositionAll and ScreenSize for configuration thisConfig
	repeat with i from 1 to (count of items in (item 1 of SavedConfigs))
		if item i of (item 1 of SavedConfigs) = thisConfig then
			set IconsAll to item i of (item 2 of SavedConfigs)
			set PositionAll to item i of (item 3 of SavedConfigs)
			set ScreenSize to item i of (item 4 of SavedConfigs)
		end if
	end repeat
	tell window "DIM" to set contents of text field "num_memorized" to ((count of IconsAll) as string)
end getConfig

on saveConfig(thisConfig) (* we want to save a configuration *)
	if (count of SavedConfigs) = 0 then (* first configuration, set up structure and save *)
		set SavedConfigs to {{thisConfig}, {IconsAll}, {PositionAll}, {ScreenSize}}
	else
		set j to 0 (* is this a new or old configuration? *)
		repeat with i from 1 to (count of items in (item 1 of SavedConfigs))
			if item i of (item 1 of SavedConfigs) = thisConfig then
				copy i to j
			end if
		end repeat
		
		if j = 0 then (* new configuration, save to end of list *)
			copy thisConfig to the end of (item 1 of SavedConfigs)
			copy IconsAll to the end of (item 2 of SavedConfigs)
			copy PositionAll to the end of (item 3 of SavedConfigs)
			copy ScreenSize to the end of (item 4 of SavedConfigs)
		else (* old configuration, replace *)
			set item j of item 2 of SavedConfigs to IconsAll
			set item j of item 3 of SavedConfigs to PositionAll
			set item j of item 4 of SavedConfigs to ScreenSize
		end if
	end if
	set contents of default entry "SavedConfigs" of user defaults to SavedConfigs --store them, damn applescript studio
end saveConfig

on deleteConfig(thisConfig)
	if (count of (item 1 of SavedConfigs)) > 1 then (* must have at least one saved configuration! should always be true, but... *)
		tell EditWindow (* freeze interface so nothing else can happen *)
			set enabled of button "done" to false
			set enabled of table view "table" of scroll view "scroll" to false
			set enabled of button "delete" to false
			set enabled of button "add" to false
		end tell
		set j to 0
		repeat with i from 1 to (count of items in (item 1 of SavedConfigs)) (* find index of config to be deleted *)
			if item i of (item 1 of SavedConfigs) = thisConfig then
				copy i to j
			end if
		end repeat
		if j = 1 then (* delete first *)
			set item 1 of SavedConfigs to rest of item 1 of SavedConfigs
			set item 2 of SavedConfigs to rest of item 2 of SavedConfigs
			set item 3 of SavedConfigs to rest of item 3 of SavedConfigs
			set item 4 of SavedConfigs to rest of item 4 of SavedConfigs
		else if j = (count of items in (item 1 of SavedConfigs)) then (* delete last *)
			set item 1 of SavedConfigs to items 1 thru -2 of item 1 of SavedConfigs
			set item 2 of SavedConfigs to items 1 thru -2 of item 2 of SavedConfigs
			set item 3 of SavedConfigs to items 1 thru -2 of item 3 of SavedConfigs
			set item 4 of SavedConfigs to items 1 thru -2 of item 4 of SavedConfigs
		else (* delete one in the middle *)
			set item 1 of SavedConfigs to items 1 thru (j - 1) of item 1 of SavedConfigs & items (j + 1) thru -1 of item 1 of SavedConfigs
			set item 2 of SavedConfigs to items 1 thru (j - 1) of item 2 of SavedConfigs & items (j + 1) thru -1 of item 2 of SavedConfigs
			set item 3 of SavedConfigs to items 1 thru (j - 1) of item 3 of SavedConfigs & items (j + 1) thru -1 of item 3 of SavedConfigs
			set item 4 of SavedConfigs to items 1 thru (j - 1) of item 4 of SavedConfigs & items (j + 1) thru -1 of item 4 of SavedConfigs
		end if
		if not (item 1 of SavedConfigs contains DefaultConfig) then (* did restore's default configuration get deleted? *)
			set DefaultConfig to item 1 of item 1 of SavedConfigs
			getConfig(DefaultConfig)
		end if
		set contents of default entry "SavedConfigs" of user defaults to SavedConfigs --store them, damn applescript studio
		set SelectedConfig to missing value
		tell EditWindow (* reset Edit window panel *)
			set enabled of button "done" to true
			set enabled of button "add" to true
			set enabled of table view "table" of scroll view "scroll" to true
			set content of table view "table" of scroll view "scroll" to item 1 of SavedConfigs
		end tell
	end if
end deleteConfig

on populatePopup(thisConfig) (* populate pop menues for Save and Restore popup buttons *)
	set contents of default entry "DefaultConfig" of user defaults to thisConfig
	tell window "DIM"
		set contents of text field "arrangement_text" to (icon_arrangement_text & thisConfig)
		
		copy title of last menu item of popup button "arrangement_pop" to mem_tl (* this is "Edit..." for english *)
		
		repeat with i from 2 to (count of items in (title of every menu item of menu of popup button "arrangement_pop"))
			delete menu item 2 of menu of popup button "arrangement_pop"
		end repeat
		repeat with i from 1 to (count of items in (item 1 of SavedConfigs))
			make new menu item at end of menu items of menu of popup button "arrangement_pop" with properties {title:(item i of (item 1 of SavedConfigs)) as string, state:((item i of (item 1 of SavedConfigs)) = thisConfig)}
		end repeat
		
		(* do the stupid seperator *)
		tell menu of popup button "arrangement_pop"
			set s to call method "createSeparatorItem:" of class "TWSeparator" with parameter it
		end tell
		
		make new menu item at end of menu items of menu of popup button "arrangement_pop" with properties {title:mem_tl} (* "Edit..." for english *)
	end tell
end populatePopup